from xml.dom import NotFoundErr

from .API import API
from .errors import *
from .utils import *

import aiohttp
import asyncio
import json
from .objects import *

class Client:
    def __init__(self,token):

        self.TOKEN = token
        self.loop = asyncio.get_event_loop()
        self.api = API()
        self.headers = {
            'Authorization': 'Bearer {}'.format(token)
        }

        self.session = aiohttp.ClientSession(loop=self.loop,headers=self.headers)

    async def request(self,url):
        async with self.session.get(url=url) as resp:
            status = resp.status
            data = await resp.json()
            
            return data, status

    async def get_player(self,tag):
        """Get a player by tag"""

        Tag = checkTag(tag)
        
        url = self.api.PLAYER.format(playerTag=Tag)

        player, status = await self.request(url)
        if 300 > status >= 200:
            c = player['club']
            if len(c) < 1:
                cl = None
            else:
                
                cl = await self.get_club(c['tag'])
                
            brs = player['brawlers']
            brrs = []
            for each in brs:
                grs = []
                srs = []
                gears = []
                for i in each['gadgets']:
                    gr = Gadget(i['name'],i['id'])
                    grs.append(gr)
                for i in each['starPowers']:
                    sr = StarPower(i['name'],i['id'])
                    srs.append(sr)
                for i in each['gears']:
                    gear = Gear(i['name'],i['id'],i['level'])
                    gears.append(gear)

                br = PlayerBrawler(each['name'],each['id'],each['power'],each['rank'],each['trophies'],each['highestTrophies'],grs,gears,srs)
                brrs.append(br)

                battleLog = await self.get_battle_log(tag)

            Pl = Player(player['name'],player['tag'],player['nameColor'],player['icon']['id'],player['trophies'],player['expLevel'],player['expPoints'],cl,player['highestTrophies'],player['soloVictories'],player['duoVictories'],player['3vs3Victories'],player['bestRoboRumbleTime'],player['bestTimeAsBigBrawler'],brrs,battleLog)

            return Pl

        elif status == 403:
            raise Forbidden(status, url, player['message'])
        elif status == 404:
            raise TagNotFoundError(status)
        elif status == 429:
            raise RateLimitError(status, url)
        elif status == 500:
            raise UnexpectedError(status, url)
        elif status == 503:
            raise ServerError(status, url)

    async def get_battle_log(self,tag):
        """Get a Player's battle log by there tag"""

        Tag = checkTag(tag)
        
        url = self.api.PLAYER.format(playerTag=Tag) + "/battlelog"

        log, status = await self.request(url)
        if 300 > status >= 200:
            return log['items']
        elif status == 403:
            raise Forbidden(status, url, log['message'])
        elif status == 404:
            raise TagNotFoundError(status)
        elif status == 429:
            raise RateLimitError(status, url)
        elif status == 500:
            raise UnexpectedError(status, url)
        elif status == 503:
            raise ServerError(status, url)

    async def get_club(self,tag):
        """Get a club by tag"""

        Tag = checkTag(tag)
        url = self.api.CLUB.format(clubTag=Tag)
        club , status= await self.request(url)

        if 300 > status >= 200:
            cc_members = []
            for each in club['members']:
                m = ClubMember(each['name'],each['icon']['id'],each['tag'],each['role'],each['nameColor'],each['trophies'])
                cc_members.append(m)

            try:
                dsc = club['description']
            except KeyError:
                dsc = None

            cl = Club(club['tag'],club['name'],dsc,club['type'],club['badgeId'],club['requiredTrophies'],club['trophies'],cc_members)

            return cl
        
        elif status == 403:
            raise Forbidden(status, url, club['message'])
        elif status == 404:
            raise TagNotFoundError(status)
        elif status == 429:
            raise RateLimitError(status, url)
        elif status == 500:
            raise UnexpectedError(status, url, club)
        elif status == 503:
            raise ServerError(status, url)

    async def brawlers(self):
        """Get a list of all the brawlers currently in the game"""
        
        url = self.api.BRAWLERS
        brawlers, status = await self.request(url)
        if 300 > status >= 200:
            brss = brawlers['items']
            brs = []
            for b in brss:
                gadgets = []
                srs = []
                
                for each in b['gadgets']:
                    gr = Gadget(each['name'],each['id'])
                    gadgets.append(gr)
                
                for each in b['starPowers']:
                    sr = StarPower(each['name'],each['id'])
                    srs.append(sr)

                brs.append(Brawler(b['name'],b['id'],srs,gadgets))

            return brs
        elif status == 403:
            raise Forbidden(status, url, brawlers['message'])
        elif status == 404:
            raise BrawlerNotFound(status)
        elif status == 429:
            raise RateLimitError(status, url)
        elif status == 500:
            raise UnexpectedError(status, url)
        elif status == 503:
            raise ServerError(status, url)

    async def events(self):
        """Get the list of all the events currently in rotation"""
        
        url = self.api.EVENTS

        events, status = await self.request(url)

        if 300 > status >= 200:
            
            List = []

            for each in events:
                l = Event(each['event']['id'],each['event']['mode'],each['event']['map'],each['startTime'],each['endTime'])

                List.append(l)

            return List

        elif status == 403:
            raise Forbidden(status, url, events['message'])
        elif status == 404:
            raise NotFoundErr(status, "Not Found!")
        elif status == 429:
            raise RateLimitError(status, url)
        elif status == 500:
            raise UnexpectedError(status, url)
        elif status == 503:
            raise ServerError(status, url)

    async def get_players_rankings(self,countryCode='global',limit=None):
        """Get top players rankings"""

        url = self.api.RANKINGS.format(countryCode=countryCode) + "/players"

        if limit:
            url += "?limit={}".format(limit)
        
        rankings, status = await self.request(url)

        if 300 > status >= 200:
        
            rankgs = []

            for each in rankings['items']:
                try:
                    cl = each['club']
                except KeyError:
                    cl = None
                else:
                    cl = cl['name']
                rankgs.append(PlayerRanking(each['name'],each['tag'],each['nameColor'],each['icon']['id'],each['trophies'],each['rank'],cl))

            return rankgs

        elif status == 403:
            raise Forbidden(status, url, rankings['message'])
        elif status == 404:
            raise NotFoundErr(status, "Not Found!")
        elif status == 429:
            raise RateLimitError(status, url)
        elif status == 500:
            raise UnexpectedError(status, url)
        elif status == 503:
            raise ServerError(status, url)

    async def get_brawlers_rankings(self,brawlerID,countryCode='global',limit=None):
        """Get top rankings based on brawlers"""

        url = self.api.RANKINGS.format(countryCode=countryCode) + "/brawlers/{}".format(brawlerID)
        
        if limit:
            url += "?limit={}".format(limit)

        rankings, status = await self.request(url)
        
        if 300 > status >= 200:
            rankgs = []

            for each in rankings['items']:
                try:
                    cl = each['club']
                except KeyError:
                    cl = None
                else:
                    cl = cl['name']
                rankgs.append(BrawlerRanking(each['tag'],each['name'],each['nameColor'],each['icon']['id'],each['trophies'],each['rank'],cl))
            
            return rankgs
        
        elif status == 403:
            raise Forbidden(status, url, rankings['message'])
        elif status == 404:
            raise BrawlerNotFound(status, brawlerID)
        elif status == 429:
            raise RateLimitError(status, url)
        elif status == 500:
            raise UnexpectedError(status, url)
        elif status == 503:
            raise ServerError(status, url)

    async def get_club_rankings(self,countryCode='global',limit=None):
        """Get top clubs rankings"""

        url = self.api.RANKINGS.format(countryCode=countryCode) + "/clubs"
    
        if limit:
            url += "?limit={}".format(limit)

        rankings, status = await self.request(url)

        if 300 > status >= 200:
            clubs = []

            for each in rankings['items']:
                clubs.append(ClubRanking(each['tag'],each['name'],each['badgeId'],each['trophies'],each['rank'],each['memberCount']))

            return clubs
        
        elif status == 403:
            raise Forbidden(status, url, rankings['message'])
        elif status == 404:
            raise CountryNotFound(status, countryCode)
        elif status == 429:
            raise RateLimitError(status, url)
        elif status == 500:
            raise UnexpectedError(status, url)
        elif status == 503:
            raise ServerError(status, url)

    async def get_club_members(self, clubTag):
        """Get members of a club"""
        
        tag = checkTag(clubTag)

        url = self.api.CLUB.format(clubTag=tag) + '/members'

        club, status = await self.request(url)

        if 300 > status >= 200:
            cc_members = []
            for each in club['items']:
                m = ClubMember(each['name'],each['icon']['id'],each['tag'],each['role'],each['nameColor'],each['trophies'])
                cc_members.append(m)

            return cc_members
        
        elif status == 403:
            raise Forbidden(status, url, club['message'])
        elif status == 404:
            raise TagNotFoundError(status)
        elif status == 429:
            raise RateLimitError(status, url)
        elif status == 500:
            raise UnexpectedError(status, url)
        elif status == 503:
            raise ServerError(status, url)
    
    async def get_brawler_byID(self, brawlerID : int):
        """Get A brawler by id"""
        
        url = self.api.BRAWLER.format(id=brawlerID)

        b, status = await self.request(url)

        if 300 > status >= 200:
            gadgets = []
            srs = []
            
            for each in b['gadgets']:
                gr = Gadget(each['name'],each['id'])
                gadgets.append(gr)
            
            for each in b['starPowers']:
                sr = StarPower(each['name'],each['id'])
                srs.append(sr)

            br = Brawler(b['name'],b['id'],srs,gadgets)

            return br

        elif status == 403:
            raise Forbidden(status, url, b['message'])
        elif status == 404:
            raise BrawlerNotFound(status, id = brawlerID)
        elif status == 429:
            raise RateLimitError(status, url)
        elif status == 500:
            raise UnexpectedError(status, url, b)
        elif status == 503:
            raise ServerError(status, url)

    async def get_brawler_byName(client, name):
        """Get a brawler by its name"""

        brawlers = await client.brawlers()

        for br in brawlers:

            if br.name.lower() == name.lower():
                return br



class Errors(Exception):
    """The Base Class for all errors"""

    def __init__(self,code,message):
        pass

    def __str__(self):
        return self.message

class Forbidden(Errors):
    """Raised If you API key is invalid"""

    def __init__(self, code, url, message):
        self.code = code
        self.url = url
        self.message = message
        super().__init__(self.code, self.message)

class TagNotFoundError(Errors):
    """Raised when a invalid player or club tag is passed"""

    def __init__(self, code, **kwargs):
        self.code = code
        self.message = "An Invalid Tag has been passed!"
        self.reason = kwargs.pop('reason',None)
        self.invalid_characters = kwargs.pop('invalid_characters', [])
        if self.reason:
            self.message += ''.join(f"\n Reason : {self.reason}")
        elif self.invalid_characters:
            self.message += ''.join(f"\n Invalid characters : {self.invalid_characters}")
        super().__init__(self.code, self.message)

class RateLimitError(Errors):
    """Raised when the rate limit is reached."""

    def __init__(self, code, url):
        self.code = code
        self.url = url
        self.message = 'The rate limit has been reached.'
        super().__init__(self.code, self.message)

class UnexpectedError(Errors):
    """Raised if an unknown error has occured."""

    def __init__(self, url, code, text):
        self.code = code
        self.url = url
        self.message = f'An unexpected error has occured.\n{text}'
        super().__init__(self.code, self.message)

class ServerError(Errors):
    """Raised if the API is down."""

    def __init__(self, code, url):
        self.code = code
        self.url = url
        self.message = 'The API is down. Please be patient and try again later.'
        super().__init__(self.code, self.message)

class BrawlerNotFound(Errors):
    """Raised when Invalid brawlerID has been passed"""

    def __init__(self,code,id = None):
        self.code = code
        if id:
            self.message = 'Invalid ID passed! {}'.format(id)

        super().__init__(self.code, self.message)

class CountryNotFound(Errors):
    """Raised when Invalid countryCode has been passed"""

    def __init__(self,code,countryCode = None):
        self.code = code
        if id:
            self.message = 'Invalid countryCode passed! {}'.format(countryCode)

        super().__init__(self.code, self.message)


from .errors import *

def checkTag(tag : str):
    tag = tag.strip("#").upper()
    allowed_chars = "0289PYLQGRJCUV"

    if len(tag) < 3:
        raise TagNotFoundError(404, reason="Tag can't be less then 3 characters!")
    invalid = [i for i in tag if i not in allowed_chars]
    if invalid:
        raise TagNotFoundError(404, reason="An Invalid character has been passed!",invalid_characters=invalid)

    if not tag.startswith("%23"):
        tag = "%23" + tag

    return tag


from .errors import *

class API:
    def __init__(self,version=1) -> None:
        
        self.BASE = 'https://api.brawlstars.com/v{}'.format(version)
        self.PLAYER = self.BASE + "/players/{playerTag}"
        self.CLUB = self.BASE + "/clubs/{clubTag}"
        self.RANKINGS = self.BASE + "/rankings/{countryCode}"
        self.BRAWLERS = self.BASE + "/brawlers"
        self.BRAWLER = self.BASE + "/brawlers/{id}"
        self.EVENTS = self.BASE + "/events/rotation"


"""
Brawl Stars API Wrapper
~~~~~~~~~~~~~~~~~~~~~~~~~
A basic wrapper for the Brawl Stars API.
:copyright: (c) 2022-Present PyStarr
:license: MIT, see LICENSE for more details.
"""

from .API import API
from .main import Client
from .objects import *

__title__ = 'brawlpy'
__author__ = 'PyStarr'
__license__ = 'MIT'
__copyright__ = 'Copyright 2022-Present PyStarr'
__version__ = '1.2.0'


brawlers = {
    "16000000" : "https://drive.google.com/uc?export=view&id=1dAvFR908593LzntFYVsJXr_um4qiFRwz",
    "16000001" : "https://drive.google.com/uc?export=view&id=1xHmMCxksAd3KWuifEpGKMVzLugYhmFvD",
    "16000002" : "https://drive.google.com/uc?export=view&id=1SOMlveroyDJEXULdJ8_WkEKWtDp74kxa",
    "16000003" : "https://drive.google.com/uc?export=view&id=10e3izEMpLRq5S_1g9REZZYFv6a_Lszg3",
    "16000004" : "https://drive.google.com/uc?export=view&id=1WqsVnqCp5BhMSaXEAdVJMFhodl4x1c6S",
    "16000005" : "https://drive.google.com/uc?export=view&id=1aW9_LnZYGbfIIAEMeRD-OUBUJ2o8zdCy",
    "16000006" : "https://drive.google.com/uc?export=view&id=14Ln4niaVoZQS5U-kxHbjb_M3FeJQ4fcv",
    "16000007" : "https://drive.google.com/uc?export=view&id=1i_XawzubrMsTwibnCleEGgILkZWJLPeJ",
    "16000008" : "https://drive.google.com/uc?export=view&id=1rob9jd_cMT1Dj_qt7RRjWegwGQca1AKX",
    "16000009" : "https://drive.google.com/uc?export=view&id=18ygqoJ2FKeKY5EqmhcBRTG5UQxry2qN_",
    "16000010" : "https://drive.google.com/uc?export=view&id=17xoy5Aw4dpJJID97eKbcX_IcaG6OA9I7",
    "16000011" : "https://drive.google.com/uc?export=view&id=1H0MZlrGvntFduEcqaSHm0TYgx5DE-Wlc",
    "16000012" : "https://drive.google.com/uc?export=view&id=16tLAAVvb_KwhrEOqPRsFTG01D_E8lKye",
    "16000013" : "https://drive.google.com/uc?export=view&id=1aQlEFbymNtxn0nkDP2GoZFDiKsMR9D4f",
    "16000014" : "https://drive.google.com/uc?export=view&id=1Fkct5iCvdjMWi-g3KwdCck1_OXs3sHlC",
    "16000015" : "https://drive.google.com/uc?export=view&id=15HiIxoF9194HOIFuxqUTQ-h5-YQSEoVG",
    "16000016" : "https://drive.google.com/uc?export=view&id=1GeRdbB_T6QH4q77V5DZq_vkXnvfkZpYk",
    "16000017" : "https://drive.google.com/uc?export=view&id=1LjGfus6pXt9A2hYEeOfBIcrcTSgiAN1_",
    "16000018" : "https://drive.google.com/uc?export=view&id=1UTB7mep0PrUT9p1MsgW25YhNjda5gsg0",
    "16000019" : "https://drive.google.com/uc?export=view&id=1EWAOzE0kNc3cw3DH7RfM-_6Jz-D9ALoo",
    "16000020" : "https://drive.google.com/uc?export=view&id=1nQVBdJb6PkdwaV566CGM21uA7ttJDhvY",
    "16000021" : "https://drive.google.com/uc?export=view&id=1mbICcY_h1YdUxvoSZIToS14JUaZvbE2l",
    "16000022" : "https://drive.google.com/uc?export=view&id=1udUF9M8ino0A3ZIGw8ptSaZC0lh9LTWs",
    "16000023" : "https://drive.google.com/uc?export=view&id=1Q2-FHGHUoNfmovn1ZObRjr_i6KJh610Q",
    "16000024" : "https://drive.google.com/uc?export=view&id=1qfWtCalsrYzQckKFOIBvNY7nDKO8IuM9",
    "16000025" : "https://drive.google.com/uc?export=view&id=1YNQMCne024WiNwklnhGsYXizuli7uoMl",
    "16000026" : "https://drive.google.com/uc?export=view&id=1X0Fp62TTlIQ6PX8gipq0XuN9MCAdFlpa",
    "16000027" : "https://drive.google.com/uc?export=view&id=1xLRF0bCblZkukyfM2h0p_N6NcPNOY77a",
    "16000028" : "https://drive.google.com/uc?export=view&id=1Hzk5vLYHSb0TVGf0LjrX0G2TRyQqaIvc",
    "16000029" : "https://drive.google.com/uc?export=view&id=14X3DhdU2XzzFU3sxOK2mIyfuqeuGIdO5",
    "16000030" : "https://drive.google.com/uc?export=view&id=1qv2PjG2e-AU2Qhe0IGm7M7F0-urKN998",
    "16000031" : "https://drive.google.com/uc?export=view&id=1U3neaBogNuy9oGY9EJqJ6wWnG_h4NRLk",
    "16000032" : "https://drive.google.com/uc?export=view&id=1XjQA6HwUkQuJMLNKyT2Drcu8xtpKYz6m",
    "16000034" : "https://drive.google.com/uc?export=view&id=1xsmR6sM71s7wwZThypyESVjvjBrN4c50",
    "16000035" : "https://drive.google.com/uc?export=view&id=15NspTtmz6xrfr7lY-ewl7-686s64Xpaf",
    "16000036" : "https://drive.google.com/uc?export=view&id=1xLvjeONLrU-wOUhxvv_FHZAqFCf03gtk",
    "16000037" : "https://drive.google.com/uc?export=view&id=1rVLeFZpVz5Mi2-1d6smUHXVSvvWJzjFU",
    "16000038" : "https://drive.google.com/uc?export=view&id=16jHS2sqJlDjy0byy4Rr3LZsWQJdfMHqJ",
    "16000039" : "https://drive.google.com/uc?export=view&id=1K7Ah2vU0BbEn1AlYeMaynqUXdHxWNfgC",
    "16000040" : "https://drive.google.com/uc?export=view&id=1V9rczuRWlZWcM-lTkNmQiPlGoAo-TkZK",
    "16000041" : "https://drive.google.com/uc?export=view&id=1PRKXASBnSw1Z6Dg9UcVn9JhLq1guZQf6",
    "16000042" : "https://drive.google.com/uc?export=view&id=1WP3X8pH_THrfXarX1gWUxyivV2h3uz3v",
    "16000043" : "https://drive.google.com/uc?export=view&id=1-AA3rab9yKtXvtiU1HW3S-vwLNjPZofO",
    "16000044" : "https://drive.google.com/uc?export=view&id=1ovyRjUNZGczg2KNIdvf-CkSjnMLdcgpL",
    "16000045" : "https://drive.google.com/uc?export=view&id=12VBQqRTEDydB6DvUHTt6EZQAoci2pGe6",
    "16000046" : "https://drive.google.com/uc?export=view&id=1idOjVsdgILT4ZFZ1Of7veixGNQYLcoEJ",
    "16000047" : "https://drive.google.com/uc?export=view&id=1O6wdsVcUsB8-sLcfodAV5xRmwVaVo9eD",
    "16000048" : "https://drive.google.com/uc?export=view&id=14Y-4BYVo2l2sn_3NEji_48FOCLNbeOKk",
    "16000049" : "https://drive.google.com/uc?export=view&id=1EBqNyrgq2-RPVYIHXTgWjOfCevsCzGfA",
    "16000050" : "https://drive.google.com/uc?export=view&id=1AQ32wTsnKM564EnGa1wEZ6n7dNnQ_L0A",
    "16000051" : "https://drive.google.com/uc?export=view&id=1XbbWDTLTs5Ms3ogqLpKyOqfEwg6k7ED0",
    "16000052" : "https://drive.google.com/uc?export=view&id=18dg1sJLDZc-pTBaKj4HTsSDkxypHfqNQ",
    "16000053" : "https://drive.google.com/uc?export=view&id=1lQ_mZd65ctx6YuhstrUBclptIvoAS6yl",
    "16000054" : "https://drive.google.com/uc?export=view&id=1Ujkutn8ovAs_9uEu9F590jhwNGCZRK-_"
}



import os
import json
from .icons import brawlers as br_icons

__location__ = os.path.realpath(
    os.path.join(os.getcwd(), os.path.dirname(__file__)))

class Player:

    """
    Player object with all its attribute

    name - The Name of the player
    tag - The Tag of the player
    nameColor - The Color of the name of the player
    icon_id - The ID of the player's icon
    trophies - The Trophies of the player
    highestTrophies - The Highest Trophies the player ever reached
    soloVictories - The number of Solo-Showdown matches the player has won
    duoVictories - The number of Duo-Showdown matches the player has won
    teamVictories - The number of 3 vs 3 matches the player has won
    bestTimeAsBigBrawler - The Player's best time as being the big brawler
    bestRoboRumbleTime - The Player's best time in Robo-Rumble mode
    brawlers - A list of all the brawlers the player has
    club - The club that the player is in
    """

    def __init__(self,name,tag,nameColor,icon_id,trophies,expLevel,expPoints,club,highestTrophies,soloVictories,duoVictories,teamVictories,bestRoboRumbleTime,bestTimeAsBigBrawler,brawlers,battleLog):
        self.name = name
        self.tag = tag
        self.nameColor = nameColor
        self.icon_id = icon_id
        self.trophies = trophies
        self.expLevel = expLevel
        self.expPoints = expPoints
        self.highestTrophies = highestTrophies
        self.soloVictories = soloVictories
        self.duoVictories = duoVictories
        self.teamVictories = teamVictories
        self.bestTimeAsBigBrawler = bestTimeAsBigBrawler
        self.bestRoboRumbleTime = bestRoboRumbleTime
        self.brawlers = brawlers
        self.club = club
        self.battleLog = battleLog

    def __repr__(self):
        return "<Player name='{0.name}' tag='{0.tag}'>".format(self)

    def __str__(self):
        return "{0.name} ({0.tag})".format(self)

class Club:

    """
    Club object with all its attribute

    name - The Name of the club
    tag - The Tag of the club
    description - The Description of the club
    type - The Type of the club (inviteOnly,closed,open)
    badgeID - The Badge ID of the club
    requiredTrophies - The amount of trophies required to join the club
    members - A list a all the members of the club
    """

    def __init__(self,tag,name,description,type,badgeID,requiredTrophies,trophies,members):
        self.tag = tag
        self.name = name
        self.description = description
        self.type = type
        self.badgeID = badgeID
        self.requiredTrophies = requiredTrophies
        self.trophies = trophies
        self.members = members

    def __repr__(self):
        return "<Club name='{0.name}' tag='{0.tag}'>".format(self)

    def __str__(self):
        return "{0.name} ({0.tag})".format(self)

class ClubMember:

    """
    ClubMember object with all its attribute

    name - The Name of the member
    tag - The Tag of the member
    icon_id - The ID of the player's icon
    role - The role of the member (president,vicePresident,senior,member)
    nameColor - The color of their name
    trophies - The total trophies of the member
    """

    def __init__(self,name,icon_id,tag,role,nameColor,trophies):
        self.name = name
        self.icon_id = icon_id
        self.tag = tag
        self.role = role
        self.nameColor = nameColor
        self.trophies = trophies
    
    def __repr__(self):
        return "<ClubMember name='{0.name}' tag='{0.tag}' role='{0.role}'>".format(self)

    def __str__(self):
        return "{0.name} ({0.tag}) => {0.role}".format(self)


class ClubRanking:

    """
    ClubRanking object with all its attribute

    name - The Name of the Club
    tag - The Tag of the Club
    badgeID - The ID of the club's badge
    trophies - The Trophies of the club
    rank - The rank of the Club
    memberCount - The amount of the member the club has
    """

    def __init__(self,tag,name,badgeID,trophies,rank,memberCount):
        self.tag = tag
        self.name = name
        self.badgeID = badgeID
        self.trophies = trophies
        self.rank = rank
        self.memberCount = memberCount

    def __repr__(self):
        return "<Club name='{0.name}' tag='{0.tag}' rank={0.rank}>".format(self)

    def __str__(self):
        return "{0.rank}. {0.name} ({0.tag})".format(self)

class PlayerRanking:

    """
    PlayerRanking object with all its attribute

    name - The Name of the player
    tag - The Tag of the player
    nameColor - The Color of the name of the player
    icon_id - The ID of the player's icon
    trophies - The Trophies of the player
    rank - The rank of the player
    clubName - The name of the club the player is in
    """

    def __init__(self,name,tag,nameColor,icon_id,trophies,rank,clubName):
        self.name = name
        self.tag = tag
        self.nameColor = nameColor
        self.icon_id = icon_id
        self.trophies = trophies
        self.rank = rank
        self.clubName = clubName

    def __repr__(self):
        return "<Club name='{0.name}' tag='{0.tag}'>".format(self)

    def __str__(self):
        return "{0.rank}. {0.name} ({0.tag})".format(self)


class BrawlerRanking:

    """
    BrawlerRanking object with all its attribute

    brawler - The brawler that the player pushed (I couldn't think of any other description lol)
    playerName - The Name of the player
    playerTag - The Tag of the player
    playerNameColor - The color of the player's name
    playerIconID - The ID of the player's icon
    trophies - The Trophies of the player
    rank - The rank of the player
    clubName - The Name of the player's Club
    """

    def __init__(self,brawler,playerTag,playerName,playerNameColor,playerIconID,trophies,rank,clubName):
        self.brawler = brawler
        self.playerTag = playerTag 
        self.playerName = playerName
        self.playerNameColor = playerNameColor
        self.playerIconID = playerIconID
        self.trophies = trophies
        self.rank = rank
        self.clubName = clubName

    def __repr__(self):
        return "<Player name='{0.playerName}' tag='{0.playerTag}' rank={0.rank} brawler={0.brawler}>".format(self)

    def __str__(self):
        return "{0.rank}. {0.playerName} ({0.playerTag}) => {0.brawler}".format(self)

class Event:

    """
    Event object with all its attribute

    id - The event ID
    mode - The mode of the event
    map - The map currently in rotation
    startTime - The time when the event got in rotation
    endTime - The time when the event will be out of rotation
    """

    def __init__(self,id,mode,map,startTime,endTime):
        self.id = id
        self.mode = mode
        self.map = map
        self.startTime = startTime
        self.endTime = endTime
    
    def __repr__(self):
        return "<Event mode='{0.mode}' map='{0.map}' id={0.id}>".format(self)

    def __str__(self):
        return "{0.mode} => {0.map} ({0.id})".format(self)

class Brawler:

    """
    Brawler object with all its attribute

    name - The name of the brawler
    id - The ID of the brawler
    icon_url = The URL of the brawler's icon
    starPowers - A list of all the starPowers the brawler has
    gadgets - A list of all the gagdets the brawler has
    """

    def __init__(self,name,id,starPowers,gadgets):
        self.name = name
        self.id = id
        self.starPowers = starPowers
        self.icon_url = self.get_icon_url()
        self.gadgets = gadgets
    
    def __repr__(self):
        return "<Brawler name='{0.name}' id='{0.id}'>".format(self)

    def __str__(self):
        return "{0.name} ({0.id})".format(self)

    def get_icon_url(self):
        
        return br_icons[str(self.id)]

class PlayerBrawler:

    """
    PlayerBrawler object with all its attribute

    name - The name of the brawler
    id - The ID of the brawler
    icon_id - The ID of the brawler's icon
    power - The power level of the brawler
    trophies - The amount of trophies the player has on the brawler
    highestTrophies - The highest amount of trophies the player ever had on the brawler
    gadgets - A list of all the gadgets the player has on this brawler
    startPowers - A list of all the star powers the player has on this brawler
    gears - A list of all the gears the player has on this brawler
    """

    def __init__(self,name,id,power,rank,trophies,highestTrophies,gadgets,gears,starPowers):
        self.name = name
        self.id = id
        self.icon_id = self.get_icon_url()
        self.power = power
        self.rank = rank
        self.trophies = trophies
        self.highestTrophies = highestTrophies
        self.gadgets = gadgets
        self.gears = gears
        self.starPowers = starPowers
    
    def __repr__(self):
        return "<Brawler name='{0.name}' rank={0.rank} trophies={0.trophies}>".format(self)

    def __str__(self):
        return "{0.name} ({0.id})".format(self)

    def get_icon_url(self):

        return br_icons[str(self.id)]


class Gadget:
    
    """
    Gadget object with all its attribute

    name - The name of the gadget
    id - The ID of the gadget
    """

    def __init__(self,name,id):
        self.name = name
        self.id = id
    
    def __repr__(self):
        return "<Gadget name='{0.name}' id='{0.id}'>".format(self)

    def __str__(self):
        return "{0.name} ({0.id})".format(self)

class StarPower:
    
    """
    StarPower object with all its attribute

    name - The name of the star power
    id - The ID of the star power
    """

    def __init__(self,name,id):
        self.name = name
        self.id = id
    
    def __repr__(self):
        return "<StarPower name='{0.name}' id='{0.id}'>".format(self)

    def __str__(self):
        return "{0.name} ({0.id})".format(self)

class Gear:
    
    """
    Gear object with all its attribute

    name - The name of the gear
    id - The ID of the gear
    level - The level of the gear
    """

    def __init__(self,name,id,level):
        self.name = name
        self.id = id
        self.level = level
    
    def __repr__(self):
        return "<Gear name='{0.name}' id='{0.id}' level={0.level}>".format(self)
    
    def __str__(self):
        return "level {0.level} gear {0.name} ({0.id})".format(self)



from setuptools import setup

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
  name = 'brawlpy',
  packages = ['brawlpy'],
  version = '1.2.5',
  license='MIT',
  description = 'A Basic wrapper for the Brawl Stars API',   
  author = 'PyStarr',
  author_email = 'pystarrorg@gmail.com',
  url = 'https://github.com/PyStarr/BrawlPy',
  long_description=long_description,
  long_description_content_type="text/markdown",
  download_url = 'https://github.com/PyStarr/BrawlPy/archive/v1.2.5.tar.gz',
  keywords = ['wrapper', 'brawlstars', 'api','brawlstarsApiWrapper'],   
  install_requires=[
          'aiohttp',
      ],
  classifiers=[
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
  ],
)